import { Injectable, ConsoleLogger } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import path from 'path';
import { Workspace } from './utility/config';
import * as fs from 'fs';
import { BlankReturnMessageDto } from './dto/ReturnMessage.dto';
import { CsvParseService } from './csv-parse/csv-parse.service';
import { Advertisement } from './utility/struct';
import { WorkspaceService } from './workspace/workspace.service';
import _ from 'lodash';

@Injectable()
export class AppService extends ConsoleLogger {
  private readonly rootPath: string;
  constructor(
    private readonly config: ConfigService,
    private csv: CsvParseService,
    private workspaceService: WorkspaceService,
  ) {
    super('app');
    this.rootPath = path.resolve(process.cwd(), this.config.get('rootPath'));
  }

  private async getAd(workspace: Workspace, name: string) {
    if (!name) {
      throw new BlankReturnMessageDto(404, 'workspace not found').toException();
    }
    try {
      const buf = await fs.promises.readFile(
        path.join(this.rootPath, `${workspace.title} - ${name}.csv`),
      );
      const ads = await this.csv.parseCsv<Advertisement>(
        buf,
        workspace.encoding || 'utf8',
        {
          columns: true,
          cast: true,
        },
      );
      return ads;
    } catch (e) {
      throw new BlankReturnMessageDto(404, 'file not found').toException();
    }
  }

  async getAdArray(workspaceName: string, name: string) {
    const workspace = this.workspaceService.getWorkspace(workspaceName);
    if (!workspace) {
      throw new BlankReturnMessageDto(404, 'file not found').toException();
    }
    const ads = await this.getAd(workspace, name);
    return _.flatten(
      ads
        .filter((ad) => ad.content)
        .map((ad) => {
          const replicaCount = ad.count || 1;
          const content = ad.content.replace(/\\n/g, '\n');
          return _.range(replicaCount).map(() => content);
        }),
    );
  }
}
